package alonzzo;

import java.time.ZonedDateTime;
import java.time.format.DateTimeFormatter;
import java.time.format.DateTimeFormatterBuilder;
import java.time.temporal.ChronoField;
import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import com.lightysoft.logmx.business.ParsedEntry;
import com.lightysoft.logmx.mgr.LogFileParser;

/**
 * Sample LogMX Parser able to parse a log file with Relative Date (nanoseconds) support.<BR/>
 * Here is an example of log file suitable for this parser:<BR/>
 <PRE>
  
 {"Subject":"Unclassified","Time":"2020-09-02T04:25:31.1924648+03:00","Level":"INFO","ClassName":"blabla","Thread":"21","Message":"yadayday"}
   
 </PRE>
 */
public class AlonzzoParser extends LogFileParser {
    /** Current parsed log entry */
    private ParsedEntry entry = null;

    /** Entry date format */
    private DateTimeFormatter dateFormat = new DateTimeFormatterBuilder() //
            .appendPattern("yyyy-MM-dd'T'HH:mm:ss") //
            .appendFraction(ChronoField.NANO_OF_SECOND, 1, 9, true).appendPattern("XXXXX") // nanoseconds, with minimum 1 and maximum 9 digits and a decimal point
            .toFormatter();

    /** Pattern for entry begin */
    private final static Pattern ENTRY_BEGIN_PATTERN = Pattern.compile(
            "^\\{\"Subject\":\"(.*?)\",\"Time\":\"(.*?)\",\"Level\":\"(.*?)\",\"ClassName\":\"(.*?)\",\"Thread\":\"(.*?)\",\"Message\":\"(.*?)\"\\}*$");

    /** Key of user-defined field "Subject" */
    private static final String EXTRA_SUBJECT_FIELD_KEY = "Subject";

    /** User-defined fields names (here, only one) */
    private static final List<String> EXTRA_FIELDS_KEYS = Arrays.asList(EXTRA_SUBJECT_FIELD_KEY);


    /** 
     * Returns the name of this parser
     * @see com.lightysoft.logmx.mgr.LogFileParser#getParserName()
     */
    @Override
    public String getParserName() {
        return "Alonzzo Parser";
    }

    /**
     * Returns the supported file type for this parser
     * @see com.lightysoft.logmx.mgr.LogFileParser#getSupportedFileType()
     */
    @Override
    public String getSupportedFileType() {
        return "Alonzzo log files";
    }

    /**
     * Process the new line of text read from file 
     * @see com.lightysoft.logmx.mgr.LogFileParser#parseLine(java.lang.String)
     */
    @Override
    protected void parseLine(String line) throws Exception {
        // If end of file, records last entry if necessary, and exits
        if (line == null) {
            recordPreviousEntryIfExists();
            return;
        }

        Matcher matcher = ENTRY_BEGIN_PATTERN.matcher(line);
        if (matcher.matches()) {
            // Record previous found entry if exists, then create a new one
            prepareNewEntry();

            entry.setDate(matcher.group(2));
            entry.setLevel(matcher.group(3));
            entry.setEmitter(matcher.group(4));
            entry.setThread(matcher.group(5));
            entry.setMessage(matcher.group(6));
            entry.getUserDefinedFields().put(EXTRA_SUBJECT_FIELD_KEY, matcher.group(1));
        }
    }

    /** 
     * Returns the ordered list of user-defined fields to display (given by their key), for each entry.
     * @see com.lightysoft.logmx.mgr.LogFileParser#getUserDefinedFields()
     */
    @Override
    public List<String> getUserDefinedFields() {
        return EXTRA_FIELDS_KEYS;
    }

    /**
     * Returns a relative Date for the given entry
     * @see com.lightysoft.logmx.mgr.LogFileParser#getRelativeEntryDate(com.lightysoft.logmx.business.ParsedEntry)
     */
    @Override
    public Date getRelativeEntryDate(ParsedEntry pEntry) throws Exception {
        return getAbsoluteEntryDate(pEntry);
    }

    /**
     * Returns the absolute Date for the given entry 
     * @see com.lightysoft.logmx.mgr.LogFileParser#getAbsoluteEntryDate(com.lightysoft.logmx.business.ParsedEntry)
     */
    @Override
    public Date getAbsoluteEntryDate(ParsedEntry pEntry) throws Exception {
        ZonedDateTime zonedDateTime = ZonedDateTime.parse(pEntry.getDate(), dateFormat);
        return Date.from(zonedDateTime.toInstant());
    }

    /**
     * Send to LogMX the current parsed log entry
     * @throws Exception
     */
    private void recordPreviousEntryIfExists() throws Exception {
        if (entry != null) {
            addEntry(entry);
        }
    }

    /**
     * Send to LogMX the current parsed log entry, then create a new one
     * @throws Exception
     */
    private void prepareNewEntry() throws Exception {
        recordPreviousEntryIfExists();
        entry = createNewEntry();
        entry.setUserDefinedFields(new HashMap<String, Object>(1)); // Create an empty Map with only one element allocated
    }
}
